package kayles;

import graphviz.GraphViz;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.event.MouseWheelEvent;
import java.awt.event.MouseWheelListener;
import java.awt.image.BufferedImage;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import javax.imageio.ImageIO;
import javax.swing.JPanel;
import utils.Range;

/**
 * A JPanel that contains a graphical representation of one configuration possibly with one reduction applied
 * @author sjoerd
 */
public class JGraph extends JPanel implements MouseWheelListener{
    
    public JGraph(){
        this.addMouseWheelListener(this);
    }
    
    Configuration c;
    Reduction r;
    
    /**
     * render this configuration
     */
    private void rerender(){
            render(c,r);
    }
    
    /**
     * Render the given configuration
     * @param c the configuration
     * @param r the reduction to draw, set this to null, to draw the base config
     */
    public void render(Configuration c, Reduction r){
      //  System.out.println("rendering...");
        this.c=c;
        this.r=r;
        try {
            GraphViz gv = new GraphViz();
            gv.addln(gv.start_graph());
            gv.addln("graph[layout=dot,ranksep=0.6];");
            gv.addln("node[fontsize=10,fontname=\"sans\"];");
            gv.add(getSource(c,r));
            gv.addln(gv.end_graph());

            byte[] data = gv.getGraph(gv.getDotSource() , "png", zoom);
            ByteArrayInputStream bis = new ByteArrayInputStream(data);
            img = ImageIO.read(bis);
            this.revalidate();
            repaint();
        } catch (IOException ex) {
            System.out.println("error: "+ex.getMessage());
        }
    }
    
    /**
     * Generate the dot source for this configuration and reduction
     * @param c
     * @param r
     * @return 
     */
    public String getSource(Configuration c,Reduction r){
        String res = "";
        
        for(Node n:c.nodes){
            res+=String.format("n%d [style=filled,fillcolor=%s,label=\"id:%d\\nred:[%s-%s]\\nwhite:[%s-%s]\"] \n",
                    c.ids.getKey(n),
                    r==null?n.type:r.get(n)==NodeType.BLACK?"GRAY":r.get(n),
                    c.ids.getKey(n),
                    n.ext_red.lo,
                    n.ext_red.hi==Range.INF?"INF":n.ext_red.hi,
                    n.ext_white.lo,
                    n.ext_white.hi==Range.INF?"INF":n.ext_white.hi
                 );
            for(Node nb:n.getNeighbors()){
                int id1 = c.ids.getKey(n);
                int id2 = c.ids.getKey(nb);
                if(id1>id2){ // only draw half of the edges
                    res+=String.format("n%d -> n%d [arrowhead=none,len=1.5];\n",id2 ,id1);
                }
            }
        }
        return res;
    }
    
    BufferedImage img;
    double zoom=1.0;
    @Override
    public void paintComponent(Graphics g){
        super.paintComponent(g);
        if(img!=null){
            int w = (int)(img.getWidth());
            int h = (int)(img.getHeight());
            g.drawImage(img, 0, 0, w, h, this);
            setPreferredSize(new Dimension(w, h));
        }else{
            g.drawString("select a configuration...",50,50);
        }
    }

    
    double rotation=0;
    @Override
    public void mouseWheelMoved(MouseWheelEvent e) {
        rotation+=e.getPreciseWheelRotation();
        zoom=Math.pow(1.2, -rotation);
        rerender();
        revalidate();
        repaint();
    }
}
