package kayles;

import java.io.Serializable;
import utils.*;
import java.util.ArrayList;

/**
 * Represents a Node in a configuration
 * @author sjoerd
 */

public class Node implements Comparable,Serializable {
    NodeType type;
    Range ext_white,ext_red; // external nodes
    private ArrayList<Node> neighbors;
    int id;
    
    /**
     * Construct a Node
     * @param id
     * @param type 
     */
    public Node(int id, NodeType type){
        this.id=id;
        this.type=type;
        ext_white = new Range();
        ext_red = new Range();
        neighbors = new ArrayList<Node>();
    }
    
    /**
     * set the external neighbors
     * @param lo_red minimum number of red neighbors
     * @param hi_red maximum number of red neighbors
     * @param lo_white minimum number of white neighbors
     * @param hi_white maximum number of white neighbors
     */
    public void setEnv(int lo_red, int hi_red, int lo_white, int hi_white){
        ext_red = new Range(lo_red, hi_red);
        ext_white = new Range(lo_white, hi_white);
    }
    
    /**
     * attach both nodes
     * @param n the other node
     */
    public void attach(Node n){
        this.add(n);
        n.add(this);
    }
    
    /**
     * attach in one direction
     * @param n the other node
     */
    public void add(Node n){
        neighbors.add(n);
    }
    
    /**
     * get the list of neighbors
     * @return 
     */
    public ArrayList<Node> getNeighbors(){
        return neighbors;
    }

    /**
     * compare this node to another node
     */
    @Override
    public int compareTo(Object o) {
        return id-((Node)o).id;
    }
    /**
     * assign an integer to this node, not unique. in an isomorphism this node 
     * must be mapped to another node with the same group number
     * @return 
     */
    public int getGroup(){
        int res =0;
        switch(type){
            case BLACK: res=0; break;
            case RED: res=1; break;
            case WHITE: res=2; break;
            case GREEN: res=3; break;
        }
        res+=(neighbors.size()<<2);
        res+=(ext_red.lo<<12);
        res+=(ext_red.hi<<16);
        res+=(ext_white.hi<<20);
        res+=(ext_white.lo<<24);   
        return res;
    }
}
