package utils;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.HashMap;

/**
 * A class that contains two HashMaps so values and keys both can be used for efficient lookups
 * @author Sjoerd Timmer
 * @param <K>
 * @param <V> 
 */
public class DualHashBidiMap<K extends Comparable,V> implements Serializable{
    HashMap<K, V> keymap;
    HashMap<V, K> valmap;
    
    public DualHashBidiMap(){
        keymap = new HashMap<K, V>();
        valmap = new HashMap<V, K>();
    }
    
    public V getValue(K key){
        return keymap.get(key);
    }
    public K getKey(V val){
        return valmap.get(val);
    }
    public void put(K key, V val){
        keymap.put(key, val);
        valmap.put(val, key);
    }
    public boolean isSet(K key){
        return keymap.containsKey(key);
    }
    
    public ArrayList<K> getKeySet(){
        return new ArrayList<K>(keymap.keySet());
    }
    
    public ArrayList<V> getValSet(){
        return new ArrayList<V>(valmap.keySet());
    }
}